import json

from betamax import BaseMatcher

from ._compat import betamax_util as util


__all__ = ('JSONBodyMatcher',)


class JSONBodyMatcher(BaseMatcher):

    """A JSON body request matcher for Betamax."""

    name = 'json-body'

    def match(self, request, recorded_request):
        """Determine if the JSON encoded bodies match."""
        recorded = util.deserialize_prepared_request(recorded_request)

        recorded_type = recorded.headers.get('Content-Type')
        request_type = request.headers.get('Content-Type')
        # Short circuit fail when the content types do not match
        if recorded_type != request_type:
            return False
        # Short circuit pass when the content type is not json
        # This permits other matchers to do the matching on this body.
        if not is_json(recorded_type):
            return True

        if request.body:
            request_json = json.loads(util.coerce_content(request.body))
        else:
            request_json = None
        recorded_json = json.loads(recorded.body) if recorded.body else None

        return request_json == recorded_json


def is_json(content_type):
    """Simplistic check for a JSON Content-Type.

    This will handle header values like::

        application/json
        application/vnd.github.v3.full+json

    """
    content_type = content_type or ''
    return content_type.startswith('application/') and 'json' in content_type



from requests_toolbelt.multipart import decoder

from betamax import BaseMatcher

from ._compat import betamax_util as util


class MultipartFormDataBodyMatcher(BaseMatcher):

    """A multipart/form-data body request matcher for Betamax."""

    name = 'multipart-form-data-body'

    def match(self, request, recorded_request):
        """Determine if the form encoded bodies match."""
        recorded = util.deserialize_prepared_request(recorded_request)

        # If neither of them have the right Content-Type set, return False
        request_content_type = request.headers.get('Content-Type')
        recorded_content_type = recorded.headers.get('Content-Type')
        if not (is_formdata(request_content_type) and
                is_formdata(recorded_content_type)):
            return False

        try:
            request_decoder = decoder.MultipartDecoder(
                request.body.encode('utf-8'),
                request_content_type
            )
            recorded_decoder = decoder.MultipartDecoder(
                recorded.body.encode('utf-8'),
                recorded_content_type
            )
        except decoder.ImproperBodyPartContentException:
            # If we can't parse on of these bodies, we probably don't want to
            # know anything about it.
            return False

        if len(request_decoder.parts) != len(recorded_decoder.parts):
            return False

        parts = zip(sorted_by_headers(request_decoder),
                    sorted_by_headers(recorded_decoder))
        for p1, p2 in parts:
            p1cd = p1.headers.get('Content-Disposition')
            p2cd = p2.headers.get('Content-Disposition')
            if (p1cd != p2cd or p1.content != p2.content):
                return False

        return True


def is_formdata(header):
    return header is not None and header.startswith('multipart/form-data')


def sorted_by_headers(decoder):
    def get_headers(part):
        return part.headers.get('Content-Disposition', '')

    return sorted(decoder.parts, key=get_headers)



"""
betamax_matchers
================

See http://betamax.rtfd.org/ for documentation.

:copyright: (c) 2014 by Ian Cordasco
:license: Apache 2.0, see LICENSE for more details

"""

__author__ = 'Ian Cordasco'
__copyright__ = 'Copyright 2014-2016 Ian Cordasco'
__license__ = 'Apache 2.0'
__title__ = 'betamax_matchers'
__version__ = '0.4.0'
__version_info__ = tuple(int(i) for i in __version__.split('.'))



"""
Implementation logic for the application/x-www-form-urlencoded request body.

To use this matcher::

    import requests

    from betamax import Betamax
    from betamax_matchers.form_urlencoded import URLEncodedBodyMatcher

    Betamax.register_request_matcher(URLEncodedBodyMatcher)

    def test_urlencoded_body():
        s = requests.Session()
        m = ['method', 'uri', 'form-urlencoded-body']
        with Betamax(s).use_cassette('urlencoded', match_requests_on=m):
            r = s.post('https://httpbin.org/post', data={
                'form-field-0': 'value-0',
                'form-field-1': 'value-1',
                'form-field-2': 'value-2',
            })

"""
try:
    from urlparse import parse_qs  # Python 2
except ImportError:
    from urllib.parse import parse_qs  # Python 3

from betamax import BaseMatcher

from ._compat import betamax_util as util

__all__ = ('URLEncodedBodyMatcher',)


class URLEncodedBodyMatcher(BaseMatcher):

    """A x-www-form-urlencoded body request matcher for Betamax."""

    name = 'form-urlencoded-body'

    def match(self, request, recorded_request):
        """Determine if the form encoded bodies match."""
        recorded = util.deserialize_prepared_request(recorded_request)

        # If neither of them have the right Content-Type set, return False
        if not (is_form(request.headers.get('Content-Type')) and
                is_form(recorded.headers.get('Content-Type'))):
            return False

        request_form = parse_qs(request.body) if request.body else None

        recorded_form = parse_qs(recorded.body) if recorded.body else None

        return request_form == recorded_form


def is_form(content_type):
    """Simplistic check for a x-www-form-urlencoded Content-Type.

    This will handle header values like::

        application/x-www-form-urlencoded

    """
    content_type = content_type or ''
    return (content_type.startswith('application/') and
            'x-www-form-urlencoded' in content_type)
